@@ -1,6 +1,6 @@
 /* $Id: D3DFeatureLevel.cpp $ */
 /** @file
- * Copyright (C) 2015 Oracle Corporation
+ * Copyright (C) 2015-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,7 +16,7 @@
  * D3D10 Feature Level and dumps it to stdout.
  */
 
-#include <Windows.h>
+#include <iprt/win/windows.h>
 #include <D3D11.h>
 #include <stdio.h>
 
