@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012 Oracle Corporation
+ * Copyright (C) 2012-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -128,10 +128,10 @@ static NTSTATUS __stdcall Resolve_RtlGetVersion(PRTL_OSVERSIONINFOW pVerInfo)
     Log(("Resolve_RtlGetVersion: %p\n", pVerInfo));
 
     RtlInitUnicodeString(&RoutineName, L"RtlGetVersion");
-    pfn = (PFNRTLGETVERSION)MmGetSystemRoutineAddress(&RoutineName);
+    pfn = (PFNRTLGETVERSION)(uintptr_t)MmGetSystemRoutineAddress(&RoutineName);
     if (!pfn)
         pfn = Fake_RtlGetVersion;
-    ASMAtomicWritePtr(&g_pfnRtlGetVersion, pfn);
+    ASMAtomicWritePtr((void * volatile *)&g_pfnRtlGetVersion, (void *)(uintptr_t)pfn);
 
     return pfn(pVerInfo);
 }
@@ -156,6 +156,7 @@ static volatile PFNPSGETPROCESSIMAGEFILENAME g_pfnPsGetProcessImageFileName = Re
 
 static LPSTR __stdcall Fake_PsGetProcessImageFileName(PEPROCESS pProcess)
 {
+    RT_NOREF(pProcess);
     Log(("Fake_PsGetProcessImageFileName: %p\n", pProcess));
     return "Fake_PsGetProcessImageFileName";
 }
@@ -168,10 +169,10 @@ static LPSTR __stdcall Resolve_PsGetProcessImageFileName(PEPROCESS pProcess)
     Log(("Resolve_PsGetProcessImageFileName: %p\n", pProcess));
 
     RtlInitUnicodeString(&RoutineName, L"PsGetProcessImageFileName");
-    pfn = (PFNPSGETPROCESSIMAGEFILENAME)MmGetSystemRoutineAddress(&RoutineName);
+    pfn = (PFNPSGETPROCESSIMAGEFILENAME)(uintptr_t)MmGetSystemRoutineAddress(&RoutineName);
     if (!pfn)
         pfn = Fake_PsGetProcessImageFileName;
-    ASMAtomicWritePtr(&g_pfnPsGetProcessImageFileName, pfn);
+    ASMAtomicWritePtr((void * volatile *)&g_pfnPsGetProcessImageFileName, (void *)(uintptr_t)pfn);
 
     return pfn(pProcess);
 }
