@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012 Oracle Corporation
+ * Copyright (C) 2012-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,7 +23,7 @@
 # include <ntstatus.h>
 # define WIN32_NO_STATUS
 #endif
-#include <intsafe.h>
+#include <iprt/win/intsafe.h>
 
 #include "VBoxCredentialProvider.h"
 
@@ -60,8 +60,7 @@ VBoxCredProvCredential::~VBoxCredProvCredential(void)
 }
 
 
-ULONG
-VBoxCredProvCredential::AddRef(void)
+ULONG VBoxCredProvCredential::AddRef(void)
 {
     LONG cRefs = InterlockedIncrement(&m_cRefs);
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::AddRef: Returning refcount=%ld\n",
@@ -70,8 +69,7 @@ VBoxCredProvCredential::AddRef(void)
 }
 
 
-ULONG
-VBoxCredProvCredential::Release(void)
+ULONG VBoxCredProvCredential::Release(void)
 {
     LONG cRefs = InterlockedDecrement(&m_cRefs);
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::Release: Returning refcount=%ld\n",
@@ -85,8 +83,7 @@ VBoxCredProvCredential::Release(void)
 }
 
 
-HRESULT
-VBoxCredProvCredential::QueryInterface(REFIID interfaceID, void **ppvInterface)
+HRESULT VBoxCredProvCredential::QueryInterface(REFIID interfaceID, void **ppvInterface)
 {
     HRESULT hr = S_OK;;
     if (ppvInterface)
@@ -125,8 +122,7 @@ VBoxCredProvCredential::QueryInterface(REFIID interfaceID, void **ppvInterface)
  * @param   fCopy                   Whether to just assign or copy the actual buffer
  *                                  contents from source -> dest.
  */
-HRESULT
-VBoxCredProvCredential::RTUTF16ToUnicode(PUNICODE_STRING pUnicodeDest, PRTUTF16 pwszSource, bool fCopy)
+HRESULT VBoxCredProvCredential::RTUTF16ToUnicode(PUNICODE_STRING pUnicodeDest, PRTUTF16 pwszSource, bool fCopy)
 {
     AssertPtrReturn(pUnicodeDest, E_POINTER);
     AssertPtrReturn(pwszSource, E_POINTER);
@@ -158,13 +154,83 @@ VBoxCredProvCredential::RTUTF16ToUnicode(PUNICODE_STRING pUnicodeDest, PRTUTF16
 }
 
 
-HRESULT
-VBoxCredProvCredential::AllocateLogonPackage(const KERB_INTERACTIVE_UNLOCK_LOGON &rUnlockLogon, PBYTE *ppPackage, DWORD *pcbPackage)
+HRESULT VBoxCredProvCredential::kerberosLogonInit(KERB_INTERACTIVE_LOGON *pLogonIn,
+                                                  CREDENTIAL_PROVIDER_USAGE_SCENARIO enmUsage,
+                                                  PRTUTF16 pwszUser, PRTUTF16 pwszPassword, PRTUTF16 pwszDomain)
 {
-    AssertPtrReturn(ppPackage, E_INVALIDARG);
-    AssertPtrReturn(pcbPackage, E_INVALIDARG);
+    AssertPtrReturn(pLogonIn,     E_INVALIDARG);
+    AssertPtrReturn(pwszUser,     E_INVALIDARG);
+    AssertPtrReturn(pwszPassword, E_INVALIDARG);
+    /* pwszDomain is optional. */
 
-    const KERB_INTERACTIVE_LOGON *pLogonIn = &rUnlockLogon.Logon;
+    HRESULT hr;
+
+    /* Do we have a domain name set? */
+    if (   pwszDomain
+        && RTUtf16Len(pwszDomain))
+    {
+        hr = RTUTF16ToUnicode(&pLogonIn->LogonDomainName, pwszDomain, true /* fCopy */);
+    }
+    else /* No domain (FQDN) given, try local computer name. */
+    {
+        WCHAR wszComputerName[MAX_COMPUTERNAME_LENGTH + 1];
+        DWORD cch = ARRAYSIZE(wszComputerName);
+        if (GetComputerNameW(wszComputerName, &cch))
+        {
+            /* Is a domain name missing? Then use the name of the local computer. */
+            hr = RTUTF16ToUnicode(&pLogonIn->LogonDomainName, wszComputerName, true /* fCopy */);
+
+            VBoxCredProvVerbose(0, "VBoxCredProvCredential::kerberosLogonInit: Local computer name=%ls\n",
+                                wszComputerName);
+        }
+        else
+            hr = HRESULT_FROM_WIN32(GetLastError());
+    }
+
+    /* Fill in the username and password. */
+    if (SUCCEEDED(hr))
+    {
+        hr = RTUTF16ToUnicode(&pLogonIn->UserName, pwszUser, true /* fCopy */);
+        if (SUCCEEDED(hr))
+        {
+            hr = RTUTF16ToUnicode(&pLogonIn->Password, pwszPassword, true /* fCopy */);
+            if (SUCCEEDED(hr))
+            {
+                /* Set credential type according to current usage scenario. */
+                switch (enmUsage)
+                {
+                    case CPUS_UNLOCK_WORKSTATION:
+                        pLogonIn->MessageType = KerbWorkstationUnlockLogon;
+                        break;
+
+                    case CPUS_LOGON:
+                        pLogonIn->MessageType = KerbInteractiveLogon;
+                        break;
+
+                    case CPUS_CREDUI:
+                        pLogonIn->MessageType = (KERB_LOGON_SUBMIT_TYPE)0; /* No message type required here. */
+                        break;
+
+                    default:
+                        VBoxCredProvVerbose(0, "VBoxCredProvCredential::kerberosLogonInit: Unknown usage scenario=%ld\n",
+                                            enmUsage);
+                        hr = E_FAIL;
+                        break;
+                }
+            }
+        }
+    }
+
+    return hr;
+}
+
+
+HRESULT VBoxCredProvCredential::kerberosLogonSerialize(const KERB_INTERACTIVE_LOGON *pLogonIn,
+                                                       PBYTE *ppPackage, DWORD *pcbPackage)
+{
+    AssertPtrReturn(pLogonIn,   E_INVALIDARG);
+    AssertPtrReturn(ppPackage,  E_INVALIDARG);
+    AssertPtrReturn(pcbPackage, E_INVALIDARG);
 
     /*
      * First, allocate enough space for the logon structure itself and separate
@@ -224,8 +290,7 @@ VBoxCredProvCredential::AllocateLogonPackage(const KERB_INTERACTIVE_UNLOCK_LOGON
  *
  * @return  HRESULT
  */
-HRESULT
-VBoxCredProvCredential::Reset(void)
+HRESULT VBoxCredProvCredential::Reset(void)
 {
 
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::Reset: Wiping credentials user=%ls, pw=%ls, domain=%ls\n",
@@ -267,8 +332,7 @@ VBoxCredProvCredential::Reset(void)
  *
  * @return  IPRT status code.
  */
-int
-VBoxCredProvCredential::RetrieveCredentials(void)
+int VBoxCredProvCredential::RetrieveCredentials(void)
 {
     int rc = VbglR3CredentialsQueryAvailability();
     if (RT_SUCCESS(rc))
@@ -377,8 +441,7 @@ VBoxCredProvCredential::RetrieveCredentials(void)
  * Initializes this credential with the current credential provider
  * usage scenario.
  */
-HRESULT
-VBoxCredProvCredential::Initialize(CREDENTIAL_PROVIDER_USAGE_SCENARIO enmUsageScenario)
+HRESULT VBoxCredProvCredential::Initialize(CREDENTIAL_PROVIDER_USAGE_SCENARIO enmUsageScenario)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::Initialize: enmUsageScenario=%ld\n", enmUsageScenario);
     m_enmUsageScenario = enmUsageScenario;
@@ -392,8 +455,7 @@ VBoxCredProvCredential::Initialize(CREDENTIAL_PROVIDER_USAGE_SCENARIO enmUsageSc
  * At the moment we only grab the credential provider events so that we can
  * trigger a re-enumeration of the credentials later.
  */
-HRESULT
-VBoxCredProvCredential::Advise(ICredentialProviderCredentialEvents *pEvents)
+HRESULT VBoxCredProvCredential::Advise(ICredentialProviderCredentialEvents *pEvents)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::Advise: pEvents=0x%p\n",
                         pEvents);
@@ -417,8 +479,7 @@ VBoxCredProvCredential::Advise(ICredentialProviderCredentialEvents *pEvents)
  *
  * We only need to release the credential provider events, if any.
  */
-HRESULT
-VBoxCredProvCredential::UnAdvise(void)
+HRESULT VBoxCredProvCredential::UnAdvise(void)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::UnAdvise\n");
 
@@ -438,8 +499,7 @@ VBoxCredProvCredential::UnAdvise(void)
  * As we don't want Winlogon to try logging in immediately we set pfAutoLogon
  * to FALSE (if set).
  */
-HRESULT
-VBoxCredProvCredential::SetSelected(PBOOL pfAutoLogon)
+HRESULT VBoxCredProvCredential::SetSelected(PBOOL pfAutoLogon)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::SetSelected\n");
 
@@ -457,8 +517,7 @@ VBoxCredProvCredential::SetSelected(PBOOL pfAutoLogon)
 /**
  * Called by LogonUI when a user profile (tile) has been unselected again.
  */
-HRESULT
-VBoxCredProvCredential::SetDeselected(void)
+HRESULT VBoxCredProvCredential::SetDeselected(void)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::SetDeselected\n");
 
@@ -474,9 +533,8 @@ VBoxCredProvCredential::SetDeselected(void)
 /**
  * Called by LogonUI to retrieve the (interactive) state of a UI field.
  */
-HRESULT
-VBoxCredProvCredential::GetFieldState(DWORD dwFieldID, CREDENTIAL_PROVIDER_FIELD_STATE *pFieldState,
-                                      CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE *pFieldstateInteractive)
+HRESULT VBoxCredProvCredential::GetFieldState(DWORD dwFieldID, CREDENTIAL_PROVIDER_FIELD_STATE *pFieldState,
+                                              CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE *pFieldstateInteractive)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetFieldState: dwFieldID=%ld\n", dwFieldID);
 
@@ -500,8 +558,7 @@ VBoxCredProvCredential::GetFieldState(DWORD dwFieldID, CREDENTIAL_PROVIDER_FIELD
  * Searches the account name based on a display (real) name (e.g. "John Doe" -> "jdoe").
  * Result "ppwszAccoutName" needs to be freed with CoTaskMemFree!
  */
-BOOL
-VBoxCredProvCredential::TranslateAccountName(PWSTR pwszDisplayName, PWSTR *ppwszAccoutName)
+BOOL VBoxCredProvCredential::TranslateAccountName(PWSTR pwszDisplayName, PWSTR *ppwszAccoutName)
 {
     AssertPtrReturn(pwszDisplayName, FALSE);
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::TranslateAccountName: Getting account name for \"%ls\" ...\n",
@@ -617,8 +674,7 @@ VBoxCredProvCredential::TranslateAccountName(PWSTR pwszDisplayName, PWSTR *ppwsz
  *
  * This might be a principal or FQDN string.
  */
-BOOL
-VBoxCredProvCredential::ExtractAccoutData(PWSTR pwszAccountData, PWSTR *ppwszAccoutName, PWSTR *ppwszDomain)
+BOOL VBoxCredProvCredential::ExtractAccoutData(PWSTR pwszAccountData, PWSTR *ppwszAccoutName, PWSTR *ppwszDomain)
 {
     AssertPtrReturn(pwszAccountData, FALSE);
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::ExtractAccoutData: Getting account name for \"%ls\" ...\n",
@@ -680,8 +736,7 @@ VBoxCredProvCredential::ExtractAccoutData(PWSTR pwszAccountData, PWSTR *ppwszAcc
  * @param   dwFieldID               Field ID to get value for.
  * @param   ppwszString             Pointer that receives the actual value of the specified field.
  */
-HRESULT
-VBoxCredProvCredential::GetStringValue(DWORD dwFieldID, PWSTR *ppwszString)
+HRESULT VBoxCredProvCredential::GetStringValue(DWORD dwFieldID, PWSTR *ppwszString)
 {
     HRESULT hr;
     if (   dwFieldID < VBOXCREDPROV_NUM_FIELDS
@@ -724,8 +779,7 @@ VBoxCredProvCredential::GetStringValue(DWORD dwFieldID, PWSTR *ppwszString)
  * @param   dwFieldID               Field ID of the submit button.
  * @param   pdwAdjacentTo           Field ID where to put the submit button next to.
  */
-HRESULT
-VBoxCredProvCredential::GetSubmitButtonValue(DWORD dwFieldID, DWORD *pdwAdjacentTo)
+HRESULT VBoxCredProvCredential::GetSubmitButtonValue(DWORD dwFieldID, DWORD *pdwAdjacentTo)
 {
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSubmitButtonValue: dwFieldID=%ld\n",
                         dwFieldID);
@@ -753,14 +807,14 @@ VBoxCredProvCredential::GetSubmitButtonValue(DWORD dwFieldID, DWORD *pdwAdjacent
  *
  * @return  HRESULT
  * @param   dwFieldID               Field to set value for.
- * @param   pcwzString              Actual value to set.
+ * @param   pwszValue               Actual value to set.
  */
-HRESULT
-VBoxCredProvCredential::SetStringValue(DWORD dwFieldID, PCWSTR pcwzString)
+HRESULT VBoxCredProvCredential::SetStringValue(DWORD dwFieldID, PCWSTR pwszValue)
 {
+    RT_NOREF(dwFieldID, pwszValue);
 #ifdef DEBUG
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::SetStringValue: dwFieldID=%ld, pcwzString=%ls\n",
-                        dwFieldID, pcwzString);
+                        dwFieldID, pwszValue);
 #endif
 
     /* Do more things here later. */
@@ -771,8 +825,7 @@ VBoxCredProvCredential::SetStringValue(DWORD dwFieldID, PCWSTR pcwzString)
 }
 
 
-HRESULT
-VBoxCredProvCredential::GetBitmapValue(DWORD dwFieldID, HBITMAP *phBitmap)
+HRESULT VBoxCredProvCredential::GetBitmapValue(DWORD dwFieldID, HBITMAP *phBitmap)
 {
     NOREF(dwFieldID);
     NOREF(phBitmap);
@@ -782,8 +835,7 @@ VBoxCredProvCredential::GetBitmapValue(DWORD dwFieldID, HBITMAP *phBitmap)
 }
 
 
-HRESULT
-VBoxCredProvCredential::GetCheckboxValue(DWORD dwFieldID, BOOL *pfChecked, PWSTR *ppwszLabel)
+HRESULT VBoxCredProvCredential::GetCheckboxValue(DWORD dwFieldID, BOOL *pfChecked, PWSTR *ppwszLabel)
 {
     NOREF(dwFieldID);
     NOREF(pfChecked);
@@ -792,8 +844,7 @@ VBoxCredProvCredential::GetCheckboxValue(DWORD dwFieldID, BOOL *pfChecked, PWSTR
 }
 
 
-HRESULT
-VBoxCredProvCredential::GetComboBoxValueCount(DWORD dwFieldID, DWORD *pcItems, DWORD *pdwSelectedItem)
+HRESULT VBoxCredProvCredential::GetComboBoxValueCount(DWORD dwFieldID, DWORD *pcItems, DWORD *pdwSelectedItem)
 {
     NOREF(dwFieldID);
     NOREF(pcItems);
@@ -802,8 +853,7 @@ VBoxCredProvCredential::GetComboBoxValueCount(DWORD dwFieldID, DWORD *pcItems, D
 }
 
 
-HRESULT
-VBoxCredProvCredential::GetComboBoxValueAt(DWORD dwFieldID, DWORD dwItem, PWSTR *ppwszItem)
+HRESULT VBoxCredProvCredential::GetComboBoxValueAt(DWORD dwFieldID, DWORD dwItem, PWSTR *ppwszItem)
 {
     NOREF(dwFieldID);
     NOREF(dwItem);
@@ -812,8 +862,7 @@ VBoxCredProvCredential::GetComboBoxValueAt(DWORD dwFieldID, DWORD dwItem, PWSTR
 }
 
 
-HRESULT
-VBoxCredProvCredential::SetCheckboxValue(DWORD dwFieldID, BOOL fChecked)
+HRESULT VBoxCredProvCredential::SetCheckboxValue(DWORD dwFieldID, BOOL fChecked)
 {
     NOREF(dwFieldID);
     NOREF(fChecked);
@@ -821,8 +870,7 @@ VBoxCredProvCredential::SetCheckboxValue(DWORD dwFieldID, BOOL fChecked)
 }
 
 
-HRESULT
-VBoxCredProvCredential::SetComboBoxSelectedValue(DWORD dwFieldId, DWORD dwSelectedItem)
+HRESULT VBoxCredProvCredential::SetComboBoxSelectedValue(DWORD dwFieldId, DWORD dwSelectedItem)
 {
     NOREF(dwFieldId);
     NOREF(dwSelectedItem);
@@ -830,8 +878,7 @@ VBoxCredProvCredential::SetComboBoxSelectedValue(DWORD dwFieldId, DWORD dwSelect
 }
 
 
-HRESULT
-VBoxCredProvCredential::CommandLinkClicked(DWORD dwFieldID)
+HRESULT VBoxCredProvCredential::CommandLinkClicked(DWORD dwFieldID)
 {
     NOREF(dwFieldID);
     return E_NOTIMPL;
@@ -847,11 +894,10 @@ VBoxCredProvCredential::CommandLinkClicked(DWORD dwFieldID)
  * @param   ppwszOptionalStatusText                 Text to set.  Optional.
  * @param   pcpsiOptionalStatusIcon                 Status icon to set.  Optional.
  */
-HRESULT
-VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE *pcpGetSerializationResponse,
-                                         CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION *pcpCredentialSerialization,
-                                         PWSTR *ppwszOptionalStatusText,
-                                         CREDENTIAL_PROVIDER_STATUS_ICON *pcpsiOptionalStatusIcon)
+HRESULT VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE *pcpGetSerializationResponse,
+                                                 CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION *pcpCredentialSerialization,
+                                                 PWSTR *ppwszOptionalStatusText,
+                                                 CREDENTIAL_PROVIDER_STATUS_ICON *pcpsiOptionalStatusIcon)
 {
     NOREF(ppwszOptionalStatusText);
     NOREF(pcpsiOptionalStatusIcon);
@@ -860,144 +906,75 @@ VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALIZATION_R
     RT_BZERO(&KerberosUnlockLogon, sizeof(KerberosUnlockLogon));
 
     /* Save a pointer to the interactive logon struct. */
-    KERB_INTERACTIVE_LOGON *pKerberosLogon = &KerberosUnlockLogon.Logon;
-    AssertPtr(pKerberosLogon);
+    KERB_INTERACTIVE_LOGON *pLogon = &KerberosUnlockLogon.Logon;
+    AssertPtr(pLogon);
 
-    HRESULT hr;
-
-#ifdef DEBUG
+#ifdef DEBUG /* Note: NEVER print this in release mode! */
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Username=%ls, Password=%ls, Domain=%ls\n",
                         m_apwszCredentials[VBOXCREDPROV_FIELDID_USERNAME],
                         m_apwszCredentials[VBOXCREDPROV_FIELDID_PASSWORD],
                         m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME]);
 #endif
 
-    /* Do we have a domain name set? */
-    if (   m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME]
-        && RTUtf16Len(m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME]))
-    {
-        hr = RTUTF16ToUnicode(&pKerberosLogon->LogonDomainName,
-                              m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME],
-                              false /* Just assign, no copy */);
-    }
-    else /* No domain (FQDN) given, try local computer name. */
-    {
-        WCHAR wszComputerName[MAX_COMPUTERNAME_LENGTH + 1];
-        DWORD cch = ARRAYSIZE(wszComputerName);
-        if (GetComputerNameW(wszComputerName, &cch))
-        {
-            /* Is a domain name missing? Then use the name of the local computer. */
-            hr = RTUTF16ToUnicode(&pKerberosLogon->LogonDomainName,
-                                  wszComputerName,
-                                  false /* Just assign, no copy */);
-
-            VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Local computer name=%ls\n",
-                                wszComputerName);
-        }
-        else
-            hr = HRESULT_FROM_WIN32(GetLastError());
-    }
-
-    /* Fill in the username and password. */
+    HRESULT hr = kerberosLogonInit(pLogon,
+                                   m_enmUsageScenario,
+                                   m_apwszCredentials[VBOXCREDPROV_FIELDID_USERNAME],
+                                   m_apwszCredentials[VBOXCREDPROV_FIELDID_PASSWORD],
+                                   m_apwszCredentials[VBOXCREDPROV_FIELDID_DOMAINNAME]);
     if (SUCCEEDED(hr))
     {
-        hr = RTUTF16ToUnicode(&pKerberosLogon->UserName,
-                              m_apwszCredentials[VBOXCREDPROV_FIELDID_USERNAME],
-                              false /* Just assign, no copy */);
+        hr = kerberosLogonSerialize(pLogon,
+                                    &pcpCredentialSerialization->rgbSerialization,
+                                    &pcpCredentialSerialization->cbSerialization);
         if (SUCCEEDED(hr))
         {
-            hr = RTUTF16ToUnicode(&pKerberosLogon->Password,
-                                  m_apwszCredentials[VBOXCREDPROV_FIELDID_PASSWORD],
-                                  false /* Just assign, no copy */);
+            HANDLE hLSA;
+            NTSTATUS s = LsaConnectUntrusted(&hLSA);
+            hr = HRESULT_FROM_NT(s);
+
             if (SUCCEEDED(hr))
             {
-                /* Set credential type according to current usage scenario. */
-                AssertPtr(pKerberosLogon);
-                switch (m_enmUsageScenario)
-                {
-                    case CPUS_UNLOCK_WORKSTATION:
-                        pKerberosLogon->MessageType = KerbWorkstationUnlockLogon;
-                        break;
-
-                    case CPUS_LOGON:
-                        pKerberosLogon->MessageType = KerbInteractiveLogon;
-                        break;
-
-                    case CPUS_CREDUI:
-                        pKerberosLogon->MessageType = (KERB_LOGON_SUBMIT_TYPE)0; /* No message type required here. */
-                        break;
-
-                    default:
-                        hr = E_FAIL;
-                        break;
-                }
-
-                if (FAILED(hr))
-                    VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Unknown usage scenario=%ld\n", m_enmUsageScenario);
-
-                if (SUCCEEDED(hr)) /* Build the logon package. */
-                {
-                    hr = AllocateLogonPackage(KerberosUnlockLogon,
-                                              &pcpCredentialSerialization->rgbSerialization,
-                                              &pcpCredentialSerialization->cbSerialization);
-                    if (FAILED(hr))
-                        VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Failed to allocate logon package, hr=0x%08x\n", hr);
-                }
-
+                LSA_STRING lsaszKerberosName;
+                size_t cchKerberosName;
+                hr = StringCchLengthA(NEGOSSP_NAME_A, USHORT_MAX, &cchKerberosName);
                 if (SUCCEEDED(hr))
                 {
-                    ULONG ulAuthPackage;
-
-                    HANDLE hLsa;
-                    NTSTATUS s = LsaConnectUntrusted(&hLsa);
-                    if (SUCCEEDED(HRESULT_FROM_NT(s)))
-                    {
-                        LSA_STRING lsaszKerberosName;
-                        size_t cchKerberosName;
-                        hr = StringCchLengthA(NEGOSSP_NAME_A, USHORT_MAX, &cchKerberosName);
-                        if (SUCCEEDED(hr))
-                        {
-                            USHORT usLength;
-                            hr = SizeTToUShort(cchKerberosName, &usLength);
-                            if (SUCCEEDED(hr))
-                            {
-                                lsaszKerberosName.Buffer        = (PCHAR)NEGOSSP_NAME_A;
-                                lsaszKerberosName.Length        = usLength;
-                                lsaszKerberosName.MaximumLength = lsaszKerberosName.Length + 1;
-
-                            }
-                        }
-
-                        if (SUCCEEDED(hr))
-                        {
-                            s = LsaLookupAuthenticationPackage(hLsa, &lsaszKerberosName,
-                                                               &ulAuthPackage);
-                            if (FAILED(HRESULT_FROM_NT(s)))
-                            {
-                                hr = HRESULT_FROM_NT(s);
-                                VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Failed looking up authentication package, hr=0x%08x\n", hr);
-                            }
-                        }
-
-                        LsaDeregisterLogonProcess(hLsa);
-                    }
-
+                    USHORT usLength;
+                    hr = SizeTToUShort(cchKerberosName, &usLength);
                     if (SUCCEEDED(hr))
                     {
-                        pcpCredentialSerialization->ulAuthenticationPackage = ulAuthPackage;
-                        pcpCredentialSerialization->clsidCredentialProvider = CLSID_VBoxCredProvider;
+                        lsaszKerberosName.Buffer        = (PCHAR)NEGOSSP_NAME_A;
+                        lsaszKerberosName.Length        = usLength;
+                        lsaszKerberosName.MaximumLength = lsaszKerberosName.Length + 1;
 
-                        /* We're done -- let the logon UI know. */
-                        *pcpGetSerializationResponse = CPGSR_RETURN_CREDENTIAL_FINISHED;
+                        ULONG ulAuthPackage = 0;
+
+                        s = LsaLookupAuthenticationPackage(hLSA, &lsaszKerberosName, &ulAuthPackage);
+                        hr = HRESULT_FROM_NT(s);
+
+                        if (SUCCEEDED(hr))
+                        {
+                            pcpCredentialSerialization->ulAuthenticationPackage = ulAuthPackage;
+                            pcpCredentialSerialization->clsidCredentialProvider = CLSID_VBoxCredProvider;
+
+                            /* We're done -- let the logon UI know. */
+                            *pcpGetSerializationResponse = CPGSR_RETURN_CREDENTIAL_FINISHED;
+                        }
+                        else
+                            VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: LsaLookupAuthenticationPackage failed with ntStatus=%ld\n", s);
                     }
                 }
+
+                LsaDeregisterLogonProcess(hLSA);
             }
             else
-                VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Error copying password, hr=0x%08x\n", hr);
+                VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: LsaConnectUntrusted failed with ntStatus=%ld\n", s);
         }
         else
-            VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization: Error copying user name, hr=0x%08x\n", hr);
+            VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: kerberosLogonSerialize failed with hr=0x%08x\n", hr);
     }
+    else
+        VBoxCredProvVerbose(1, "VBoxCredProvCredential::GetSerialization: kerberosLogonInit failed with hr=0x%08x\n", hr);
 
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::GetSerialization returned hr=0x%08x\n", hr);
     return hr;
@@ -1016,12 +993,12 @@ VBoxCredProvCredential::GetSerialization(CREDENTIAL_PROVIDER_GET_SERIALIZATION_R
  * @param   ppwszOptionalStatusText     Pointer that receives the optional status text.
  * @param   pcpsiOptionalStatusIcon     Pointer that receives the optional status icon.
  */
-HRESULT
-VBoxCredProvCredential::ReportResult(NTSTATUS ntStatus,
-                                     NTSTATUS ntSubStatus,
-                                     PWSTR *ppwszOptionalStatusText,
-                                     CREDENTIAL_PROVIDER_STATUS_ICON *pcpsiOptionalStatusIcon)
+HRESULT VBoxCredProvCredential::ReportResult(NTSTATUS ntStatus,
+                                             NTSTATUS ntSubStatus,
+                                             PWSTR *ppwszOptionalStatusText,
+                                             CREDENTIAL_PROVIDER_STATUS_ICON *pcpsiOptionalStatusIcon)
 {
+    RT_NOREF(ntStatus, ntSubStatus, ppwszOptionalStatusText, pcpsiOptionalStatusIcon);
     VBoxCredProvVerbose(0, "VBoxCredProvCredential::ReportResult: ntStatus=%ld, ntSubStatus=%ld\n",
                         ntStatus, ntSubStatus);
     return S_OK;
