@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012 Oracle Corporation
+ * Copyright (C) 2012-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,17 +22,14 @@
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <Windows.h>
+#include <iprt/win/windows.h>
 #include <NTSecAPI.h>
 #define SECURITY_WIN32
 #include <Security.h>
 #include <ShlGuid.h>
 #include <strsafe.h>
 
-#pragma warning(push)
-#pragma warning(disable : 4995)
-#include <Shlwapi.h>
-#pragma warning(pop)
+#include <iprt/win/shlwapi.h>
 
 #include <iprt/string.h>
 
@@ -76,7 +73,7 @@ public:
     IFACEMETHODIMP GetComboBoxValueAt(DWORD dwFieldID, DWORD dwItem, PWSTR *ppwszItem);
     IFACEMETHODIMP GetSubmitButtonValue(DWORD dwFieldID, DWORD *pdwAdjacentTo);
 
-    IFACEMETHODIMP SetStringValue(DWORD dwFieldID, PCWSTR pcwzString);
+    IFACEMETHODIMP SetStringValue(DWORD dwFieldID, PCWSTR pwszValue);
     IFACEMETHODIMP SetCheckboxValue(DWORD dwFieldID, BOOL fChecked);
     IFACEMETHODIMP SetComboBoxSelectedValue(DWORD dwFieldID, DWORD dwSelectedItem);
     IFACEMETHODIMP CommandLinkClicked(DWORD dwFieldID);
@@ -97,8 +94,10 @@ public:
 
 protected:
     HRESULT RTUTF16ToUnicode(PUNICODE_STRING pUnicodeDest, PRTUTF16 pwszSource, bool fCopy);
-    HRESULT AllocateLogonPackage(const KERB_INTERACTIVE_UNLOCK_LOGON &rUnlockLogon,
-                                 BYTE **ppPackage, DWORD *pcbPackage);
+    HRESULT kerberosLogonInit(KERB_INTERACTIVE_LOGON *pLogonIn,
+                              CREDENTIAL_PROVIDER_USAGE_SCENARIO enmUsage,
+                              PRTUTF16 pwszUser, PRTUTF16 pwszPassword, PRTUTF16 pwszDomain);
+    HRESULT kerberosLogonSerialize(const KERB_INTERACTIVE_LOGON *pLogon, PBYTE *ppPackage, DWORD *pcbPackage);
 
 private:
     /** Internal reference count. */
