@@ -35,6 +35,12 @@
 #include <AudioToolbox/AudioConverter.h>
 #include <AudioToolbox/AudioToolbox.h>
 
+/* Audio Queue buffer configuration. */
+#define AQ_BUF_COUNT    32      /* Number of buffers. */
+#define AQ_BUF_SIZE     512     /* Size of each buffer in bytes. */
+#define AQ_BUF_TOTAL    (AQ_BUF_COUNT * AQ_BUF_SIZE)
+#define AQ_BUF_SAMPLES  (AQ_BUF_TOTAL / 4)  /* Hardcoded 4 bytes per sample! */
+
 #if 0
 # include <iprt/file.h>
 # define DEBUG_DUMP_PCM_DATA
@@ -276,7 +282,7 @@ typedef struct COREAUDIOSTREAM
     /** The actual audio queue being used. */
     AudioQueueRef               audioQueue;
     /** The audio buffers which are used with the above audio queue. */
-    AudioQueueBufferRef         audioBuffer[3];
+    AudioQueueBufferRef         audioBuffer[AQ_BUF_COUNT];
     /** The acquired (final) audio format for this stream. */
     AudioStreamBasicDescription asbdStream;
     /** The device' UUID. */
@@ -759,7 +765,7 @@ static DECLCALLBACK(int) coreAudioQueueThread(RTTHREAD hThreadSelf, void *pvUser
     if (err != noErr)
         return VERR_GENERAL_FAILURE; /** @todo Fudge! */
 
-    const size_t cbBufSize = _4K; /** @todo Make this configurable! */
+    const size_t cbBufSize = AQ_BUF_SIZE; /** @todo Make this configurable! */
 
     /*
      * Allocate audio buffers.
@@ -2022,7 +2028,7 @@ static DECLCALLBACK(int) drvHostCoreAudioInitIn(PPDMIHOSTAUDIO pInterface,
     rc = coreAudioStreamInitQueue(pStreamIn->cbCtx.pStream, true /* fIn */, pCfgReq, pCfgAcq);
     if (RT_SUCCESS(rc))
     {
-        *pcSamples = _4K; /** @todo Make this configurable. */
+        *pcSamples = AQ_BUF_SAMPLES;
     }
     if (RT_SUCCESS(rc))
     {
@@ -2128,7 +2134,7 @@ static DECLCALLBACK(int) drvHostCoreAudioInitOut(PPDMIHOSTAUDIO pInterface,
     rc = coreAudioStreamInitQueue(pStreamOut->cbCtx.pStream, false /* fIn */, pCfgReq, pCfgAcq);
     if (RT_SUCCESS(rc))
     {
-        *pcSamples = _4K; /** @todo Make this configurable. */
+        *pcSamples = AQ_BUF_SAMPLES;
     }
     if (RT_SUCCESS(rc))
     {
