@@ -1,12 +1,10 @@
 /* $Id: UIMultiScreenLayout.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIMultiScreenLayout class implementation
+ * VBox Qt GUI - UIMultiScreenLayout class implementation.
  */
 
 /*
- * Copyright (C) 2010-2013 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,51 +15,44 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Qt includes: */
-#include <QApplication>
-#include <QDesktopWidget>
-#include <QMenu>
+# include <QApplication>
+# include <QMenu>
 
 /* GUI includes: */
-#include "UIDefs.h"
-#include "UIMultiScreenLayout.h"
-#include "UIActionPoolRuntime.h"
-#include "UIMachineLogic.h"
-#include "UIFrameBuffer.h"
-#include "UISession.h"
-#include "UIMessageCenter.h"
-#include "VBoxGlobal.h"
+# include "UIDefs.h"
+# include "UIMultiScreenLayout.h"
+# include "UIActionPoolRuntime.h"
+# include "UIMachineLogic.h"
+# include "UIFrameBuffer.h"
+# include "UISession.h"
+# include "UIMessageCenter.h"
+# include "UIExtraDataManager.h"
+# include "UIDesktopWidgetWatchdog.h"
+# include "VBoxGlobal.h"
 
 /* COM includes: */
-#include "COMEnums.h"
-#include "CSession.h"
-#include "CConsole.h"
-#include "CMachine.h"
-#include "CDisplay.h"
+# include "COMEnums.h"
+# include "CSession.h"
+# include "CConsole.h"
+# include "CMachine.h"
+# include "CDisplay.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 
 UIMultiScreenLayout::UIMultiScreenLayout(UIMachineLogic *pMachineLogic)
     : m_pMachineLogic(pMachineLogic)
-    , m_pViewMenu(0)
 {
     /* Calculate host/guest screen count: */
     calculateHostMonitorCount();
     calculateGuestScreenCount();
 }
 
-UIMultiScreenLayout::~UIMultiScreenLayout()
-{
-    /* Cleanup view-menu: */
-    cleanupViewMenu();
-}
-
-void UIMultiScreenLayout::setViewMenu(QMenu *pViewMenu)
-{
-    /* Assign view-menu: */
-    m_pViewMenu = pViewMenu;
-    /* Prepare view-menu: */
-    prepareViewMenu();
-}
-
 void UIMultiScreenLayout::update()
 {
     LogRelFlow(("UIMultiScreenLayout::update: Started...\n"));
@@ -77,11 +68,9 @@ void UIMultiScreenLayout::update()
     /* Load all combinations stored in the settings file.
      * We have to make sure they are valid, which means there have to be unique combinations
      * and all guests screens need there own host screen. */
-    CMachine machine = m_pMachineLogic->session().GetMachine();
     CDisplay display = m_pMachineLogic->session().GetConsole().GetDisplay();
-    bool fShouldWeAutoMountGuestScreens = VBoxGlobal::shouldWeAutoMountGuestScreens(machine, false);
-    LogRelFlow(("UIMultiScreenLayout::update: GUI/AutomountGuestScreens is %s.\n", fShouldWeAutoMountGuestScreens ? "enabled" : "disabled"));
-    QDesktopWidget *pDW = QApplication::desktop();
+    bool fShouldWeAutoMountGuestScreens = gEDataManager->autoMountGuestScreensEnabled(vboxGlobal().managedVMUuid());
+    LogRel(("GUI: UIMultiScreenLayout::update: GUI/AutomountGuestScreens is %s\n", fShouldWeAutoMountGuestScreens ? "enabled" : "disabled"));
     foreach (int iGuestScreen, m_guestScreens)
     {
         /* Initialize variables: */
@@ -91,34 +80,28 @@ void UIMultiScreenLayout::update()
         if (!fValid)
         {
             /* If the user ever selected a combination in the view menu, we have the following entry: */
-            QString strTest = machine.GetExtraData(QString("%1%2").arg(GUI_VirtualScreenToHostScreen).arg(iGuestScreen));
-            bool fOk;
-            /* Check is this value can be converted: */
-            iHostScreen = strTest.toInt(&fOk);
+            iHostScreen = gEDataManager->hostScreenForPassedGuestScreen(iGuestScreen, vboxGlobal().managedVMUuid());
             /* Revalidate: */
-            fValid =    fOk /* Valid data */
-                     && iHostScreen >= 0 && iHostScreen < m_cHostScreens /* In the host screen bounds? */
+            fValid =    iHostScreen >= 0 && iHostScreen < m_cHostScreens /* In the host screen bounds? */
                      && m_screenMap.key(iHostScreen, -1) == -1; /* Not taken already? */
         }
 
         if (!fValid)
         {
             /* Check the position of the guest window in normal mode.
-             * This makes sure that on first use the window opens on the same screen as the normal window was before.
-             * This even works with multi-screen. The user just have to move all the normal windows to the target screens
-             * and they will magically open there in seamless/fullscreen also. */
-            QString strTest1 = machine.GetExtraData(GUI_LastNormalWindowPosition + (iGuestScreen > 0 ? QString::number(iGuestScreen): ""));
-            QRegExp posParser("(-?\\d+),(-?\\d+),(-?\\d+),(-?\\d+)");
-            if (posParser.exactMatch(strTest1))
+             * This makes sure that on first use fullscreen/seamless window opens on the same host-screen as the normal window was before.
+             * This even works with multi-screen. The user just have to move all the normal windows to the target host-screens
+             * and they will magically open there in fullscreen/seamless also. */
+            QRect geo = gEDataManager->machineWindowGeometry(UIVisualStateType_Normal, iGuestScreen, vboxGlobal().managedVMUuid());
+            /* If geometry is valid: */
+            if (!geo.isNull())
             {
-                /* If parsing was successfully, convert it to a position: */
-                bool fOk1, fOk2;
-                QPoint p(posParser.cap(1).toInt(&fOk1), posParser.cap(2).toInt(&fOk2));
-                /* Check to which screen the position belongs: */
-                iHostScreen = pDW->screenNumber(p);
+                /* Get top-left corner position: */
+                QPoint topLeftPosition(geo.topLeft());
+                /* Check which host-screen the position belongs to: */
+                iHostScreen = gpDesktop->screenNumber(topLeftPosition);
                 /* Revalidate: */
-                fValid =    fOk1 && fOk2 /* Valid data */
-                         && iHostScreen >= 0 && iHostScreen < m_cHostScreens /* In the host screen bounds? */
+                fValid =    iHostScreen >= 0 && iHostScreen < m_cHostScreens /* In the host screen bounds? */
                          && m_screenMap.key(iHostScreen, -1) == -1; /* Not taken already? */
             }
         }
@@ -145,8 +128,9 @@ void UIMultiScreenLayout::update()
         else if (fShouldWeAutoMountGuestScreens)
         {
             /* Then we have to disable excessive guest-screen: */
-            LogRelFlow(("UIMultiScreenLayout::update: Disabling excessive guest-screen %d.\n", iGuestScreen));
+            LogRel(("GUI: UIMultiScreenLayout::update: Disabling excessive guest-screen %d\n", iGuestScreen));
             display.SetVideoModeHint(iGuestScreen, false, false, 0, 0, 0, 0, 0);
+            m_pMachineLogic->uisession()->setScreenVisibleHostDesires(iGuestScreen, false);
         }
     }
 
@@ -177,14 +161,15 @@ void UIMultiScreenLayout::update()
                 pFrameBuffer->setAutoEnabled(true);
             }
             /* Re-enable guest-screen with proper resolution: */
-            LogRelFlow(("UIMultiScreenLayout::update: Enabling guest-screen %d with following resolution: %dx%d.\n",
-                        iGuestScreen, uWidth, uHeight));
+            LogRel(("GUI: UIMultiScreenLayout::update: Enabling guest-screen %d with following resolution: %dx%d\n",
+                    iGuestScreen, uWidth, uHeight));
             display.SetVideoModeHint(iGuestScreen, true, false, 0, 0, uWidth, uHeight, 32);
+            m_pMachineLogic->uisession()->setScreenVisibleHostDesires(iGuestScreen, true);
         }
     }
 
-    /* Update menu actions: */
-    updateMenuActions(false);
+    /* Notifies about layout update: */
+    emit sigScreenLayoutUpdate();
 
     LogRelFlow(("UIMultiScreenLayout::update: Finished!\n"));
 }
@@ -196,8 +181,6 @@ void UIMultiScreenLayout::rebuild()
     /* Recalculate host/guest screen count: */
     calculateHostMonitorCount();
     calculateGuestScreenCount();
-    /* Update view-menu: */
-    prepareViewMenu();
     /* Update layout: */
     update();
 
@@ -229,31 +212,8 @@ quint64 UIMultiScreenLayout::memoryRequirements() const
     return memoryRequirements(m_screenMap);
 }
 
-bool UIMultiScreenLayout::isHostTaskbarCovert() const
+void UIMultiScreenLayout::sltHandleScreenLayoutChange(int iRequestedGuestScreen, int iRequestedHostScreen)
 {
-    /* Check for all screens which are in use if they have some
-     * taskbar/menubar/dock on it. Its done by comparing the available with the
-     * screen geometry. Only if they are the same for all screens, there are no
-     * host area covert. This is a little bit ugly, but there seems no other
-     * way to find out if we are on a screen where the taskbar/dock or whatever
-     * is present. */
-    QDesktopWidget *pDW = QApplication::desktop();
-    for (int i = 0; i < m_screenMap.size(); ++i)
-    {
-        int hostScreen = m_screenMap.value(i);
-        if (pDW->availableGeometry(hostScreen) != pDW->screenGeometry(hostScreen))
-            return true;
-    }
-    return false;
-}
-
-void UIMultiScreenLayout::sltScreenLayoutChanged(QAction *pAction)
-{
-    /* Parse incoming information: */
-    int a = pAction->data().toInt();
-    int iRequestedGuestScreen = RT_LOWORD(a);
-    int iRequestedHostScreen = RT_HIWORD(a);
-
     /* Search for the virtual screen which is currently displayed on the
      * requested host screen. When there is one found, we swap both. */
     QMap<int,int> tmpMap(m_screenMap);
@@ -267,7 +227,7 @@ void UIMultiScreenLayout::sltScreenLayoutChanged(QAction *pAction)
     /* Check the memory requirements first: */
     bool fSuccess = true;
     CMachine machine = m_pMachineLogic->session().GetMachine();
-    if (m_pMachineLogic->uisession()->isGuestAdditionsActive())
+    if (m_pMachineLogic->uisession()->isGuestSupportsGraphics())
     {
         quint64 availBits = machine.GetVRAMSize() * _1M * 8;
         quint64 usedBits = memoryRequirements(tmpMap);
@@ -278,7 +238,7 @@ void UIMultiScreenLayout::sltScreenLayoutChanged(QAction *pAction)
             if (m_pMachineLogic->visualStateType() == UIVisualStateType_Seamless)
                 msgCenter().cannotSwitchScreenInSeamless((((usedBits + 7) / 8 + _1M - 1) / _1M) * _1M);
             else
-                fSuccess = msgCenter().cannotSwitchScreenInFullscreen((((usedBits + 7) / 8 + _1M - 1) / _1M) * _1M) != QIMessageBox::Cancel;
+                fSuccess = msgCenter().cannotSwitchScreenInFullscreen((((usedBits + 7) / 8 + _1M - 1) / _1M) * _1M);
         }
     }
     /* Make sure memory requirements matched: */
@@ -287,19 +247,17 @@ void UIMultiScreenLayout::sltScreenLayoutChanged(QAction *pAction)
 
     /* Swap the maps: */
     m_screenMap = tmpMap;
-    /* Update menu actions: */
-    updateMenuActions(true);
-    /* Inform the observer: */
-    emit sigScreenLayoutChanged();
+
+    /* Save guest-to-host mapping: */
+    saveScreenMapping();
+
+    /* Notifies about layout change: */
+    emit sigScreenLayoutChange();
 }
 
 void UIMultiScreenLayout::calculateHostMonitorCount()
 {
-#if (QT_VERSION >= 0x040600)
-    m_cHostScreens = QApplication::desktop()->screenCount();
-#else /* (QT_VERSION >= 0x040600) */
-    m_cHostScreens = QApplication::desktop()->numScreens();
-#endif /* !(QT_VERSION >= 0x040600) */
+    m_cHostScreens = gpDesktop->screenCount();
 }
 
 void UIMultiScreenLayout::calculateGuestScreenCount()
@@ -316,84 +274,12 @@ void UIMultiScreenLayout::calculateGuestScreenCount()
             m_disabledGuestScreens << iGuestScreen;
 }
 
-void UIMultiScreenLayout::prepareViewMenu()
+void UIMultiScreenLayout::saveScreenMapping()
 {
-    /* Make sure view-menu was set: */
-    if (!m_pViewMenu)
-        return;
-
-    /* Cleanup menu first: */
-    cleanupViewMenu();
-
-    /* If we do have more than one host/guest screen: */
-    if (m_cHostScreens > 1 || m_guestScreens.size() > 1)
+    foreach (const int &iGuestScreen, m_guestScreens)
     {
-        m_pViewMenu->addSeparator();
-        foreach (int iGuestScreen, m_guestScreens)
-        {
-            m_screenMenuList << m_pViewMenu->addMenu(tr("Virtual Screen %1").arg(iGuestScreen + 1));
-            m_screenMenuList.last()->menuAction()->setData(true);
-            QActionGroup *pScreenGroup = new QActionGroup(m_screenMenuList.last());
-            pScreenGroup->setExclusive(true);
-            connect(pScreenGroup, SIGNAL(triggered(QAction*)), this, SLOT(sltScreenLayoutChanged(QAction*)));
-            for (int a = 0; a < m_cHostScreens; ++a)
-            {
-                QAction *pAction = pScreenGroup->addAction(tr("Use Host Screen %1").arg(a + 1));
-                pAction->setCheckable(true);
-                pAction->setData(RT_MAKE_U32(iGuestScreen, a));
-            }
-            m_screenMenuList.last()->addActions(pScreenGroup->actions());
-        }
-    }
-
-    /* Update menu actions: */
-    updateMenuActions(false);
-}
-
-void UIMultiScreenLayout::cleanupViewMenu()
-{
-    /* Make sure view-menu was set: */
-    if (!m_pViewMenu)
-        return;
-
-    /* Cleanup view-menu actions: */
-    while (!m_screenMenuList.isEmpty())
-        delete m_screenMenuList.takeFirst();
-}
-
-void UIMultiScreenLayout::updateMenuActions(bool fWithSave)
-{
-    /* Make sure view-menu was set: */
-    if (!m_pViewMenu)
-        return;
-
-    /* Get the list of all view-menu actions: */
-    QList<QAction*> viewMenuActions = gActionPool->action(UIActionIndexRuntime_Menu_View)->menu()->actions();
-    /* Get the list of all view related actions: */
-    QList<QAction*> viewActions;
-    for (int i = 0; i < viewMenuActions.size(); ++i)
-        if (viewMenuActions[i]->data().toBool())
-            viewActions << viewMenuActions[i];
-    /* Update view actions: */
-    CMachine machine = m_pMachineLogic->session().GetMachine();
-    for (int iViewAction = 0; iViewAction < viewActions.size(); ++iViewAction)
-    {
-        int iGuestScreen = m_guestScreens[iViewAction];
-        int iHostScreen = m_screenMap.value(iGuestScreen, -1);
-        if (fWithSave)
-        {
-            QString strHostScreen(iHostScreen != -1 ? QString::number(iHostScreen) : QString());
-            machine.SetExtraData(QString("%1%2").arg(GUI_VirtualScreenToHostScreen).arg(iViewAction), strHostScreen);
-        }
-        QList<QAction*> screenActions = viewActions.at(iViewAction)->menu()->actions();
-        /* Update screen actions: */
-        for (int j = 0; j < screenActions.size(); ++j)
-        {
-            QAction *pTmpAction = screenActions.at(j);
-            pTmpAction->blockSignals(true);
-            pTmpAction->setChecked(RT_HIWORD(pTmpAction->data().toInt()) == iHostScreen);
-            pTmpAction->blockSignals(false);
-        }
+        const int iHostScreen = m_screenMap.value(iGuestScreen, -1);
+        gEDataManager->setHostScreenForPassedGuestScreen(iGuestScreen, iHostScreen, vboxGlobal().managedVMUuid());
     }
 }
 
@@ -402,16 +288,19 @@ quint64 UIMultiScreenLayout::memoryRequirements(const QMap<int, int> &screenLayo
     ULONG width = 0;
     ULONG height = 0;
     ULONG guestBpp = 0;
+    LONG xOrigin = 0;
+    LONG yOrigin = 0;
     quint64 usedBits = 0;
     CDisplay display = m_pMachineLogic->uisession()->session().GetConsole().GetDisplay();
     foreach (int iGuestScreen, m_guestScreens)
     {
         QRect screen;
         if (m_pMachineLogic->visualStateType() == UIVisualStateType_Seamless)
-            screen = QApplication::desktop()->availableGeometry(screenLayout.value(iGuestScreen, 0));
+            screen = gpDesktop->availableGeometry(screenLayout.value(iGuestScreen, 0));
         else
-            screen = QApplication::desktop()->screenGeometry(screenLayout.value(iGuestScreen, 0));
-        display.GetScreenResolution(iGuestScreen, width, height, guestBpp);
+            screen = gpDesktop->screenGeometry(screenLayout.value(iGuestScreen, 0));
+        KGuestMonitorStatus monitorStatus = KGuestMonitorStatus_Enabled;
+        display.GetScreenResolution(iGuestScreen, width, height, guestBpp, xOrigin, yOrigin, monitorStatus);
         usedBits += screen.width() * /* display width */
                     screen.height() * /* display height */
                     guestBpp + /* guest bits per pixel */
