@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,13 +19,12 @@
 #define __UIMachineSettingsStorage_h__
 
 /* Qt includes: */
-#include <QtGlobal> /* for Q_WS_MAC */
-#ifdef Q_WS_MAC
+#ifdef VBOX_WS_MAC
 /* Somewhere Carbon.h includes AssertMacros.h which defines the macro "check".
  * In QItemDelegate a class method is called "check" also. As we not used the
  * macro undefine it here. */
 # undef check
-#endif /* Q_WS_MAC */
+#endif /* VBOX_WS_MAC */
 #include <QItemDelegate>
 #include <QPointer>
 
@@ -82,6 +81,9 @@ enum PixmapType
     USBControllerNormal,
     USBControllerExpand,
     USBControllerCollapse,
+    NVMeControllerNormal,
+    NVMeControllerExpand,
+    NVMeControllerCollapse,
     FloppyControllerNormal,
     FloppyControllerExpand,
     FloppyControllerCollapse,
@@ -94,6 +96,8 @@ enum PixmapType
     SCSIControllerAddDis,
     USBControllerAddEn,
     USBControllerAddDis,
+    NVMeControllerAddEn,
+    NVMeControllerAddDis,
     FloppyControllerAddEn,
     FloppyControllerAddDis,
 
@@ -225,6 +229,19 @@ private:
     uint size() const;
 };
 
+/* NVMe Controller Type */
+class NVMeStorageControllerType : public AbstractControllerType
+{
+public:
+
+    NVMeStorageControllerType (KStorageControllerType aSubType);
+
+private:
+
+    KStorageControllerType first() const;
+    uint size() const;
+};
+
 /* Abstract Item */
 class AbstractItem
 {
@@ -394,7 +411,6 @@ private:
 
     StorageSlot mAttSlot;
     QString mAttMediumId;
-    bool mAttIsShowDiffs;
     bool mAttIsHostDrive;
     bool mAttIsPassthrough;
     bool mAttIsTempEject;
@@ -439,6 +455,7 @@ public:
         R_IsMoreFloppyControllersPossible,
         R_IsMoreSASControllersPossible,
         R_IsMoreUSBControllersPossible,
+        R_IsMoreNVMeControllersPossible,
         R_IsMoreAttachmentsPossible,
 
         R_CtrName,
@@ -707,6 +724,7 @@ private slots:
     void addFloppyController();
     void addSASController();
     void addUSBController();
+    void addNVMeController();
     void delController();
 
     void addAttachment();
@@ -788,6 +806,7 @@ private:
     QAction *mAddSASCtrAction;
     QAction *mAddFloppyCtrAction;
     QAction *mAddUSBCtrAction;
+    QAction *mAddNVMeCtrAction;
     QAction *mAddAttAction;
     QAction *mDelAttAction;
     QAction *mAddHDAttAction;
