@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <VBox/vmm/vm.h>
 #include <VBox/vmm/vmm.h>
 #include <VBox/vmm/cpum.h>
@@ -47,9 +47,10 @@
 
 #include <signal.h>
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 static volatile bool g_fSignaled = false;
 
 
@@ -126,7 +127,39 @@ static int scriptDtrReg(PVM pVM, char *pszVar, char *pszValue, void *pvUser)
 
 
 
-
+/* variables - putting in global scope to avoid MSC warning C4640.  */
+static struct
+{
+    const char *pszVar;
+    int (*pfnHandler)(PVM pVM, char *pszVar, char *pszValue, void *pvUser);
+    PFNRT pvUser;
+} g_aVars[] =
+{
+    { "eax", scriptGPReg,  (PFNRT)CPUMSetGuestEAX },
+    { "ebx", scriptGPReg,  (PFNRT)CPUMSetGuestEBX },
+    { "ecx", scriptGPReg,  (PFNRT)CPUMSetGuestECX },
+    { "edx", scriptGPReg,  (PFNRT)CPUMSetGuestEDX },
+    { "esp", scriptGPReg,  (PFNRT)CPUMSetGuestESP },
+    { "ebp", scriptGPReg,  (PFNRT)CPUMSetGuestEBP },
+    { "esi", scriptGPReg,  (PFNRT)CPUMSetGuestESI },
+    { "edi", scriptGPReg,  (PFNRT)CPUMSetGuestEDI },
+    { "efl", scriptGPReg,  (PFNRT)CPUMSetGuestEFlags },
+    { "eip", scriptGPReg,  (PFNRT)CPUMSetGuestEIP },
+    { "ss",  scriptSelReg, (PFNRT)CPUMSetGuestSS },
+    { "cs",  scriptSelReg, (PFNRT)CPUMSetGuestCS },
+    { "ds",  scriptSelReg, (PFNRT)CPUMSetGuestDS },
+    { "es",  scriptSelReg, (PFNRT)CPUMSetGuestES },
+    { "fs",  scriptSelReg, (PFNRT)CPUMSetGuestFS },
+    { "gs",  scriptSelReg, (PFNRT)CPUMSetGuestGS },
+    { "cr0", scriptSysReg, (PFNRT)CPUMSetGuestCR0 },
+    { "cr2", scriptSysReg, (PFNRT)CPUMSetGuestCR2 },
+    { "cr3", scriptSysReg, (PFNRT)CPUMSetGuestCR3 },
+    { "cr4", scriptSysReg, (PFNRT)CPUMSetGuestCR4 },
+    { "ldtr",scriptSelReg, (PFNRT)CPUMSetGuestLDTR },
+    { "tr",  scriptSelReg, (PFNRT)CPUMSetGuestTR },
+    { "idtr",scriptDtrReg, (PFNRT)CPUMSetGuestIDTR },
+    { "gdtr",scriptDtrReg, (PFNRT)CPUMSetGuestGDTR }
+};
 
 
 static int scriptCommand(PVM pVM, const char *pszIn, size_t cch)
@@ -144,46 +177,12 @@ static int scriptCommand(PVM pVM, const char *pszIn, size_t cch)
         RTStrStripR(psz);
         char *pszValue = RTStrStrip(pszEqual + 1);
 
-        /* variables */
-        static struct
-        {
-            const char *pszVar;
-            int (*pfnHandler)(PVM pVM, char *pszVar, char *pszValue, void *pvUser);
-            PFNRT pvUser;
-        } aVars[] =
-        {
-            { "eax", scriptGPReg,  (PFNRT)CPUMSetGuestEAX },
-            { "ebx", scriptGPReg,  (PFNRT)CPUMSetGuestEBX },
-            { "ecx", scriptGPReg,  (PFNRT)CPUMSetGuestECX },
-            { "edx", scriptGPReg,  (PFNRT)CPUMSetGuestEDX },
-            { "esp", scriptGPReg,  (PFNRT)CPUMSetGuestESP },
-            { "ebp", scriptGPReg,  (PFNRT)CPUMSetGuestEBP },
-            { "esi", scriptGPReg,  (PFNRT)CPUMSetGuestESI },
-            { "edi", scriptGPReg,  (PFNRT)CPUMSetGuestEDI },
-            { "efl", scriptGPReg,  (PFNRT)CPUMSetGuestEFlags },
-            { "eip", scriptGPReg,  (PFNRT)CPUMSetGuestEIP },
-            { "ss",  scriptSelReg, (PFNRT)CPUMSetGuestSS },
-            { "cs",  scriptSelReg, (PFNRT)CPUMSetGuestCS },
-            { "ds",  scriptSelReg, (PFNRT)CPUMSetGuestDS },
-            { "es",  scriptSelReg, (PFNRT)CPUMSetGuestES },
-            { "fs",  scriptSelReg, (PFNRT)CPUMSetGuestFS },
-            { "gs",  scriptSelReg, (PFNRT)CPUMSetGuestGS },
-            { "cr0", scriptSysReg, (PFNRT)CPUMSetGuestCR0 },
-            { "cr2", scriptSysReg, (PFNRT)CPUMSetGuestCR2 },
-            { "cr3", scriptSysReg, (PFNRT)CPUMSetGuestCR3 },
-            { "cr4", scriptSysReg, (PFNRT)CPUMSetGuestCR4 },
-            { "ldtr",scriptSelReg, (PFNRT)CPUMSetGuestLDTR },
-            { "tr",  scriptSelReg, (PFNRT)CPUMSetGuestTR },
-            { "idtr",scriptDtrReg, (PFNRT)CPUMSetGuestIDTR },
-            { "gdtr",scriptDtrReg, (PFNRT)CPUMSetGuestGDTR }
-        };
-
         rc = -1;
-        for (unsigned i = 0; i < RT_ELEMENTS(aVars); i++)
+        for (unsigned i = 0; i < RT_ELEMENTS(g_aVars); i++)
         {
-            if (!strcmp(psz, aVars[i].pszVar))
+            if (!strcmp(psz, g_aVars[i].pszVar))
             {
-                rc = aVars[i].pfnHandler(pVM, psz, pszValue, (void*)(uintptr_t)aVars[i].pvUser);
+                rc = g_aVars[i].pfnHandler(pVM, psz, pszValue, (void *)(uintptr_t)g_aVars[i].pvUser);
                 break;
             }
         }
@@ -309,8 +308,8 @@ static DECLCALLBACK(int) loadMem(PVM pVM, RTFILE File, uint64_t *poff)
             }
 
             /* Write that page to the guest - skip known rom areas for now. */
-            if (GCPhys < 0xa0000 || GCPhys >= 0x10000) /* ASSUME size of a8Page is a power of 2. */
-                PGMPhysWrite(pVM, GCPhys, &au8Page, cbRead);
+            if (GCPhys < 0xa0000 || GCPhys >= 0x100000) /* ASSUME size of a8Page is a power of 2. */
+                PGMPhysWrite(pVM, GCPhys, &au8Page, cbRead, PGMACCESSORIGIN_DEBUGGER);
             GCPhys += cbRead;
         }
     }
@@ -328,8 +327,9 @@ static DECLCALLBACK(int) loadMem(PVM pVM, RTFILE File, uint64_t *poff)
  * @returns VBox status code.
  * @param   pVM     Pointer to the VM.
  */
-static DECLCALLBACK(int) cfgmR3CreateDefault(PVM pVM, void *pvUser)
+static DECLCALLBACK(int) cfgmR3CreateDefault(PUVM pUVM, PVM pVM, void *pvUser)
 {
+    RT_NOREF1(pUVM);
     uint64_t cbMem = *(uint64_t *)pvUser;
     int rc;
     int rcAll = VINF_SUCCESS;
@@ -618,8 +618,12 @@ static void syntax(void)
 }
 
 
-int main(int argc, char **argv)
+/**
+ *  Entry point.
+ */
+extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char **envp)
 {
+    RT_NOREF1(envp);
     int rcRet = 1;
     int rc;
     RTR3InitExe(argc, &argv, RTR3INIT_FLAGS_SUPLIB);
@@ -831,25 +835,26 @@ int main(int argc, char **argv)
      * Create empty VM.
      */
     PVM pVM;
-    rc = VMR3Create(1, NULL, NULL, NULL, cfgmR3CreateDefault, &cbMem, &pVM);
+    PUVM pUVM;
+    rc = VMR3Create(1, NULL, NULL, NULL, cfgmR3CreateDefault, &cbMem, &pVM, &pUVM);
     if (RT_SUCCESS(rc))
     {
         /*
          * Load memory.
          */
         if (FileRawMem != NIL_RTFILE)
-            rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)loadMem, 3, pVM, FileRawMem, &offRawMem);
+            rc = VMR3ReqCallWaitU(pUVM, VMCPUID_ANY, (PFNRT)loadMem, 3, pVM, FileRawMem, &offRawMem);
         else
-            rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)SSMR3Load,
-                                 7, pVM, pszSavedState, (uintptr_t)NULL /*pStreamOps*/, (uintptr_t)NULL /*pvUser*/,
-                                 SSMAFTER_DEBUG_IT, (uintptr_t)NULL /*pfnProgress*/, (uintptr_t)NULL /*pvProgressUser*/);
+            rc = VMR3ReqCallWaitU(pUVM, VMCPUID_ANY, (PFNRT)SSMR3Load,
+                                  7, pVM, pszSavedState, (uintptr_t)NULL /*pStreamOps*/, (uintptr_t)NULL /*pvUser*/,
+                                  SSMAFTER_DEBUG_IT, (uintptr_t)NULL /*pfnProgress*/, (uintptr_t)NULL /*pvProgressUser*/);
         if (RT_SUCCESS(rc))
         {
             /*
              * Load register script.
              */
             if (FileScript != NIL_RTFILE)
-                rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)scriptRun, 2, pVM, FileScript);
+                rc = VMR3ReqCallWaitU(pUVM, VMCPUID_ANY, (PFNRT)scriptRun, 2, pVM, FileScript);
             if (RT_SUCCESS(rc))
             {
                 if (fPowerOn)
@@ -859,7 +864,7 @@ int main(int argc, char **argv)
                      */
                     if (u32WarpDrive != 100)
                     {
-                        rc = TMR3SetWarpDrive(pVM, u32WarpDrive);
+                        rc = TMR3SetWarpDrive(pUVM, u32WarpDrive);
                         if (RT_FAILURE(rc))
                             RTPrintf("warning: TMVirtualSetWarpDrive(,%u) -> %Rrc\n", u32WarpDrive, rc);
                     }
@@ -877,11 +882,11 @@ int main(int argc, char **argv)
 #endif
                     if (RT_SUCCESS(rc))
                     {
-                        rc = EMR3SetExecutionPolicy(pVM, EMEXECPOLICY_RECOMPILE_RING0, true); AssertReleaseRC(rc);
-                        rc = EMR3SetExecutionPolicy(pVM, EMEXECPOLICY_RECOMPILE_RING3, true); AssertReleaseRC(rc);
-                        DBGFR3Info(pVM, "cpumguest", "verbose", NULL);
+                        rc = EMR3SetExecutionPolicy(pUVM, EMEXECPOLICY_RECOMPILE_RING0, true); AssertReleaseRC(rc);
+                        rc = EMR3SetExecutionPolicy(pUVM, EMEXECPOLICY_RECOMPILE_RING3, true); AssertReleaseRC(rc);
+                        DBGFR3Info(pUVM, "cpumguest", "verbose", NULL);
                         if (fPowerOn)
-                            rc = VMR3PowerOn(pVM);
+                            rc = VMR3PowerOn(pUVM);
                         if (RT_SUCCESS(rc))
                         {
                             RTPrintf("info: VM is running\n");
@@ -901,7 +906,7 @@ int main(int argc, char **argv)
                      * Don't start it, just enter the debugger.
                      */
                     RTPrintf("info: entering debugger...\n");
-                    DBGFR3Info(pVM, "cpumguest", "verbose", NULL);
+                    DBGFR3Info(pUVM, "cpumguest", "verbose", NULL);
                     signal(SIGINT, SigInterrupt);
                     while (!g_fSignaled)
                         RTThreadSleep(1000);
@@ -917,12 +922,14 @@ int main(int argc, char **argv)
         /*
          * Cleanup.
          */
-        rc = VMR3Destroy(pVM);
+        rc = VMR3Destroy(pUVM);
         if (!RT_SUCCESS(rc))
         {
             RTPrintf("tstAnimate: error: failed to destroy vm! rc=%Rrc\n", rc);
             rcRet++;
         }
+
+        VMR3ReleaseUVM(pUVM);
     }
     else
     {
@@ -933,3 +940,14 @@ int main(int argc, char **argv)
     return rcRet;
 }
 
+
+#if !defined(VBOX_WITH_HARDENING) || !defined(RT_OS_WINDOWS)
+/**
+ * Main entry point.
+ */
+int main(int argc, char **argv, char **envp)
+{
+    return TrustedMain(argc, argv, envp);
+}
+#endif
+
