@@ -170,7 +170,7 @@ my_wildcard()
     else
         MY_FOLDER="$1-All.lst"
     fi
-    EXCLUDES="*.log;*.kup;*~;*.pyc;*.exe;*.sys;*.dll;*.o;*.obj;*.lib;*.a;*.ko;*.class;*.cvsignore;*.done;*.project;*.actionScriptProperties;*.scm-settings;*.svn-base;.svn/*"
+    EXCLUDES="*.log;*.kup;*~;*.pyc;*.exe;*.sys;*.dll;*.o;*.obj;*.lib;*.a;*.ko;*.class;*.cvsignore;*.done;*.project;*.actionScriptProperties;*.scm-settings;*.svnpatch.rej;*.svn-base;.svn/*;*.gitignore"
     echo '        <F N="'"${2}"'/*" Recurse="1" Excludes="'"${EXCLUDES}"'"/>' >> "${MY_FOLDER}"
 }
 
@@ -296,6 +296,99 @@ my_generate_folder()
         "${MY_FILE}-Testcases.lst" "${MY_FILE}-Others.lst"
 }
 
+##
+# Function generating a project build config.
+#
+# @param    $1      The project file name.
+# @param    $2      Build config name.
+# @param    $3      Extra kBuild command line options, variant 1.
+# @param    $4      Extra kBuild command line options, variant 2.
+# @param    $4+     Include directories.
+# @param    $N      --end-includes
+my_generate_project_config()
+{
+    MY_FILE="${1}";
+    MY_CFG_NAME="${2}";
+    MY_KMK_EXTRAS1="${3}";
+    MY_KMK_EXTRAS2="${4}";
+    MY_KMK_EXTRAS3="${5}";
+    MY_KMK_EXTRAS4="${6}";
+    shift; shift; shift; shift; shift; shift;
+
+    echo '    <Config Name="'"${MY_CFG_NAME}"'" OutputFile="" CompilerConfigName="Latest Version">'             >> "${MY_FILE}"
+    echo '        <Menu>'                                                                                       >> "${MY_FILE}"
+
+    echo '            <Target Name="Compile" MenuCaption="&amp;Compile" CaptureOutputWith="ProcessBuffer"'      >> "${MY_FILE}"
+    echo '                    SaveOption="SaveCurrent" RunFromDir="%p" ClearProcessBuffer="1">'                 >> "${MY_FILE}"
+    echo -n '                <Exec CmdLine="'"${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS1}"' -C %p %n.o'              >> "${MY_FILE}"
+    if test -n "${MY_KMK_EXTRAS2}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS2} -C %p %n.o" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS3}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS3} -C %p %n.o" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS4}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS4} -C %p %n.o" >> "${MY_FILE}"
+    fi
+    echo  '"/>' >> "${MY_FILE}"
+    echo '            </Target>'                                                                                >> "${MY_FILE}"
+
+    echo '            <Target Name="Build" MenuCaption="&amp;Build"CaptureOutputWith="ProcessBuffer"'           >> "${MY_FILE}"
+    echo '                    SaveOption="SaveWorkspaceFiles" RunFromDir="%rw" ClearProcessBuffer="1">'         >> "${MY_FILE}"
+    echo -n '                <Exec CmdLine="'"${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS1}"' -C %rw'                  >> "${MY_FILE}"
+    if test -n "${MY_KMK_EXTRAS2}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS2} -C %rw" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS3}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS3} -C %rw" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS4}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS4} -C %rw" >> "${MY_FILE}"
+    fi
+    echo  '"/>' >> "${MY_FILE}"
+    echo '            </Target>'                                                                                >> "${MY_FILE}"
+
+    echo '            <Target Name="Rebuild" MenuCaption="&amp;Rebuild" CaptureOutputWith="ProcessBuffer"'      >> "${MY_FILE}"
+    echo '                    SaveOption="SaveWorkspaceFiles" RunFromDir="%rw" ClearProcessBuffer="1">'         >> "${MY_FILE}"
+    echo -n '                <Exec CmdLine="'"${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS1}"' -C %rw'                  >> "${MY_FILE}"
+    if test -n "${MY_KMK_EXTRAS2}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS2} -C %rw rebuild" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS3}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS3} -C %rw rebuild" >> "${MY_FILE}"
+    fi
+    if test -n "${MY_KMK_EXTRAS4}"; then
+        echo -n " && ${MY_KMK_INVOCATION} ${MY_KMK_EXTRAS4} -C %rw rebuild" >> "${MY_FILE}"
+    fi
+    echo  '"/>' >> "${MY_FILE}"
+    echo '            </Target>'                                                                                >> "${MY_FILE}"
+
+    #echo '            <Target Name="Debug" MenuCaption="&amp;Debug" SaveOption="SaveNone" RunFromDir="%rw">'    >> "${MY_FILE}"
+    #echo '                <Exec/>'                                                                              >> "${MY_FILE}"
+    #echo '            </Target>'                                                                                >> "${MY_FILE}"
+    #echo '            <Target Name="Execute" MenuCaption="E&amp;xecute" SaveOption="SaveNone" RunFromDir="%rw">'>> "${MY_FILE}"
+    #echo '                <Exec/>'                                                                              >> "${MY_FILE}"
+    #echo '            </Target>'                                                                                >> "${MY_FILE}"
+    echo '        </Menu>'                                                                                      >> "${MY_FILE}"
+
+    #
+    # Include directories.
+    #
+    echo '        <Includes>'                                                                                   >> "${MY_FILE}"
+    while test $# -ge 1  -a  "${1}" != "--end-includes";
+    do
+        for f in $1;
+        do
+            my_abs_dir ${f}
+            echo '            <Include Dir="'"${MY_ABS_DIR}/"'"/>'                                              >> "${MY_FILE}"
+        done
+        shift
+    done
+    shift
+    echo '        </Includes>'                                                                                  >> "${MY_FILE}"
+    echo '    </Config>'                                                                                        >> "${MY_FILE}"
+}
+
 
 ##
 # Function generating a project.
@@ -333,45 +426,31 @@ my_generate_project()
     my_abs_dir "${MY_WRK_DIR}"                                                                                          >> "${MY_FILE}"
     echo '        WorkingDir="'"${MY_ABS_DIR}"'"'                                                                       >> "${MY_FILE}"
     echo '        >'                                                                                                    >> "${MY_FILE}"
-    echo '    <Config Name="Release" OutputFile="" CompilerConfigName="Latest Version">'                                >> "${MY_FILE}"
-    echo '        <Menu>'                                                                                               >> "${MY_FILE}"
-    echo '            <Target Name="Compile" MenuCaption="&amp;Compile" CaptureOutputWith="ProcessBuffer"'              >> "${MY_FILE}"
-    echo '                    SaveOption="SaveCurrent" RunFromDir="%p" ClearProcessBuffer="1">'                         >> "${MY_FILE}"
-    echo '                <Exec CmdLine="'"${MY_KMK_INVOCATION}"' -C %p %n.o"/>'                                        >> "${MY_FILE}"
-    echo '            </Target>'                                                                                        >> "${MY_FILE}"
-    echo '            <Target Name="Build" MenuCaption="&amp;Build"CaptureOutputWith="ProcessBuffer"'                   >> "${MY_FILE}"
-    echo '                    SaveOption="SaveWorkspaceFiles" RunFromDir="%rw" ClearProcessBuffer="1">'                 >> "${MY_FILE}"
-    echo '                <Exec CmdLine="'"${MY_KMK_INVOCATION}"' -C %rw"/>'                                            >> "${MY_FILE}"
-    echo '            </Target>'                                                                                        >> "${MY_FILE}"
-    echo '            <Target Name="Rebuild" MenuCaption="&amp;Rebuild" CaptureOutputWith="ProcessBuffer"'              >> "${MY_FILE}"
-    echo '                    SaveOption="SaveWorkspaceFiles" RunFromDir="%rw" ClearProcessBuffer="1">'                 >> "${MY_FILE}"
-    echo '                <Exec CmdLine="'"${MY_KMK_INVOCATION}"' -C %rw rebuild"/>'                                    >> "${MY_FILE}"
-    echo '            </Target>'                                                                                        >> "${MY_FILE}"
-    echo '            <Target Name="Debug" MenuCaption="&amp;Debug" SaveOption="SaveNone" RunFromDir="%rw">'            >> "${MY_FILE}"
-    echo '                <Exec/>'                                                                                      >> "${MY_FILE}"
-    echo '            </Target>'                                                                                        >> "${MY_FILE}"
-    echo '            <Target Name="Execute" MenuCaption="E&amp;xecute" SaveOption="SaveNone" RunFromDir="%rw">'        >> "${MY_FILE}"
-    echo '                <Exec/>'                                                                                      >> "${MY_FILE}"
-    echo '            </Target>'                                                                                        >> "${MY_FILE}"
-    echo '        </Menu>'                                                                                              >> "${MY_FILE}"
+    my_generate_project_config "${MY_FILE}" "Default" "" "" "" "" $*
+    my_generate_project_config "${MY_FILE}" "Debug + hardening"     "KBUILD_TYPE=debug VBOX_WITH_HARDENING=1"   "" "" "" $*
+    my_generate_project_config "${MY_FILE}" "Release + hardening"   "KBUILD_TYPE=release VBOX_WITH_HARDENING=1" "" "" "" $*
+    my_generate_project_config "${MY_FILE}" "Debug+Release + hardening" \
+        "KBUILD_TYPE=debug VBOX_WITH_HARDENING=1" \
+        "KBUILD_TYPE=release VBOX_WITH_HARDENING=1" \
+        "" "" $*
+    my_generate_project_config "${MY_FILE}" "Debug w/o hardening"   "KBUILD_TYPE=debug VBOX_WITHOUT_HARDENING=1"   "" "" $*
+    my_generate_project_config "${MY_FILE}" "Release w/o hardening" "KBUILD_TYPE=release VBOX_WITHOUT_HARDENING=1" "" "" $*
+    my_generate_project_config "${MY_FILE}" "Debug+Release w/o hardening" \
+        "KBUILD_TYPE=debug VBOX_WITHOUT_HARDENING=1" \
+        "KBUILD_TYPE=release VBOX_WITHOUT_HARDENING=1" \
+        "" "" $*
+    my_generate_project_config "${MY_FILE}" "Debug+Release with and without hardening" \
+        "KBUILD_TYPE=debug VBOX_WITH_HARDENING=1" \
+        "KBUILD_TYPE=release VBOX_WITH_HARDENING=1" \
+        "KBUILD_TYPE=debug VBOX_WITHOUT_HARDENING=1" \
+        "KBUILD_TYPE=release VBOX_WITHOUT_HARDENING=1" \
+        $*
 
-    #
-    # Include directories.
-    #
-    echo '        <Includes>'                                                                                           >> "${MY_FILE}"
     while test $# -ge 1  -a  "${1}" != "--end-includes";
     do
-        for f in $1;
-        do
-            my_abs_dir ${f}
-            echo '            <Include Dir="'"${MY_ABS_DIR}/"'"/>'                                                      >> "${MY_FILE}"
-        done
-        shift
-    done
-    shift
-    echo '        </Includes>'                                                                                          >> "${MY_FILE}"
-    echo '    </Config>'                                                                                                >> "${MY_FILE}"
-
+        shift;
+    done;
+    shift;
 
     #
     # Process directories+files and create folders.
@@ -632,6 +711,124 @@ EOF
 #define RTASN1TYPE_STANDARD_PROTOTYPES_NO_GET_CORE(a_TypeNm, a_DeclMacro, a_ImplExtNm) int  a_ImplExtNm##_Init(P##a_TypeNm pThis, PCRTASN1ALLOCATORVTABLE pAllocator); int  a_ImplExtNm##_Clone(P##a_TypeNm pThis, PC##a_TypeNm) pSrc, PCRTASN1ALLOCATORVTABLE pAllocator); void a_ImplExtNm##_Delete(P##a_TypeNm pThis); int  a_ImplExtNm##_Enum(P##a_TypeNm pThis, PFNRTASN1ENUMCALLBACK pfnCallback, uint32_t uDepth, void *pvUser); int  a_ImplExtNm##_Compare(PC##a_TypeNm) pLeft, PC##a_TypeNm pRight); int  a_ImplExtNm##_DecodeAsn1(PRTASN1CURSOR pCursor, uint32_t fFlags, P##a_TypeNm pThis, const char *pszErrorTag); int  a_ImplExtNm##_CheckSanity(PC##a_TypeNm pThis, uint32_t fFlags, PRTERRINFO pErrInfo, const char *pszErrorTag)
 #define RTASN1TYPE_STANDARD_PROTOTYPES(a_TypeNm, a_DeclMacro, a_ImplExtNm, a_Asn1CoreNm) inline PRTASN1CORE a_ImplExtNm##_GetAsn1Core(PC##a_TypeNm pThis) { return (PRTASN1CORE)&pThis->a_Asn1CoreNm; } inline bool a_ImplExtNm##_IsPresent(PC##a_TypeNm pThis) { return pThis && RTASN1CORE_IS_PRESENT(&pThis->a_Asn1CoreNm); } RTASN1TYPE_STANDARD_PROTOTYPES_NO_GET_CORE(a_TypeNm, a_DeclMacro, a_ImplExtNm)
 
+#define BS3_DECL(type)                  type
+#define BS3_DECL_CALLBACK(type)         type
+#define TMPL_NM(name)                   name##_mmm
+#define TMPL_FAR_NM(name)               name##_mmm_far
+#define BS3_CMN_NM(name)                name
+#define BS3_CMN_FAR_NM(name)            name
+#define BS3_CMN_FN_NM(name)             name
+#define BS3_DATA_NM(name)               name
+#define BS3_FAR
+#define BS3_FAR_CODE
+#define BS3_FAR_DATA
+#define BS3_NEAR
+#define BS3_NEAR_CODE
+#define BS3_CMN_PROTO_STUB(a_RetType, a_Name, a_Params) a_RetType a_Name a_Params
+#define BS3_CMN_PROTO_NOSB(a_RetType, a_Name, a_Params) a_RetType a_Name a_Params
+#define BS3_CMN_DEF(       a_RetType, a_Name, a_Params) a_RetType a_Name a_Params
+#define BS3_MODE_PROTO_STUB(a_RetType, a_Name, a_Params) \
+    a_RetType a_Name##_mmm           a_Params; \
+    a_RetType a_Name##_mmm_far       a_Params; \
+    a_RetType a_Name##_rm            a_Params; \
+    a_RetType a_Name##_pe16          a_Params; \
+    a_RetType a_Name##_pe16_32       a_Params; \
+    a_RetType a_Name##_pe16_v86      a_Params; \
+    a_RetType a_Name##_pe32          a_Params; \
+    a_RetType a_Name##_pe32_16       a_Params; \
+    a_RetType a_Name##_pev86         a_Params; \
+    a_RetType a_Name##_pp16          a_Params; \
+    a_RetType a_Name##_pp16_32       a_Params; \
+    a_RetType a_Name##_pp16_v86      a_Params; \
+    a_RetType a_Name##_pp32          a_Params; \
+    a_RetType a_Name##_pp32_16       a_Params; \
+    a_RetType a_Name##_ppv86         a_Params; \
+    a_RetType a_Name##_pae16         a_Params; \
+    a_RetType a_Name##_pae16_32      a_Params; \
+    a_RetType a_Name##_pae16_v86     a_Params; \
+    a_RetType a_Name##_pae32         a_Params; \
+    a_RetType a_Name##_pae32_16      a_Params; \
+    a_RetType a_Name##_paev86        a_Params; \
+    a_RetType a_Name##_lm16          a_Params; \
+    a_RetType a_Name##_lm32          a_Params; \
+    a_RetType a_Name##_lm64          a_Params; \
+    a_RetType a_Name##_rm_far        a_Params; \
+    a_RetType a_Name##_pe16_far      a_Params; \
+    a_RetType a_Name##_pe16_v86_far  a_Params; \
+    a_RetType a_Name##_pe32_16_far   a_Params; \
+    a_RetType a_Name##_pev86_far     a_Params; \
+    a_RetType a_Name##_pp16_far      a_Params; \
+    a_RetType a_Name##_pp16_v86_far  a_Params; \
+    a_RetType a_Name##_pp32_16_far   a_Params; \
+    a_RetType a_Name##_ppv86_far     a_Params; \
+    a_RetType a_Name##_pae16_far     a_Params; \
+    a_RetType a_Name##_pae16_v86_far a_Params; \
+    a_RetType a_Name##_pae32_16_far  a_Params; \
+    a_RetType a_Name##_paev86_far    a_Params; \
+    a_RetType a_Name##_lm16_far      a_Params
+#define BS3_MODE_PROTO_NOSB(a_RetType, a_Name, a_Params) \
+    a_RetType a_Name##_mmm           a_Params; \
+    a_RetType a_Name##_mmm_far       a_Params; \
+    a_RetType a_Name##_rm            a_Params; \
+    a_RetType a_Name##_pe16          a_Params; \
+    a_RetType a_Name##_pe16_32       a_Params; \
+    a_RetType a_Name##_pe16_v86      a_Params; \
+    a_RetType a_Name##_pe32          a_Params; \
+    a_RetType a_Name##_pe32_16       a_Params; \
+    a_RetType a_Name##_pev86         a_Params; \
+    a_RetType a_Name##_pp16          a_Params; \
+    a_RetType a_Name##_pp16_32       a_Params; \
+    a_RetType a_Name##_pp16_v86      a_Params; \
+    a_RetType a_Name##_pp32          a_Params; \
+    a_RetType a_Name##_pp32_16       a_Params; \
+    a_RetType a_Name##_ppv86         a_Params; \
+    a_RetType a_Name##_pae16         a_Params; \
+    a_RetType a_Name##_pae16_32      a_Params; \
+    a_RetType a_Name##_pae16_v86     a_Params; \
+    a_RetType a_Name##_pae32         a_Params; \
+    a_RetType a_Name##_pae32_16      a_Params; \
+    a_RetType a_Name##_paev86        a_Params; \
+    a_RetType a_Name##_lm16          a_Params; \
+    a_RetType a_Name##_lm32          a_Params; \
+    a_RetType a_Name##_lm64          a_Params; \
+    a_RetType a_Name##_rm_far        a_Params; \
+    a_RetType a_Name##_pe16_far      a_Params; \
+    a_RetType a_Name##_pe16_v86_far  a_Params; \
+    a_RetType a_Name##_pe32_16_far   a_Params; \
+    a_RetType a_Name##_pev86_far     a_Params; \
+    a_RetType a_Name##_pp16_far      a_Params; \
+    a_RetType a_Name##_pp16_v86_far  a_Params; \
+    a_RetType a_Name##_pp32_16_far   a_Params; \
+    a_RetType a_Name##_ppv86_far     a_Params; \
+    a_RetType a_Name##_pae16_far     a_Params; \
+    a_RetType a_Name##_pae16_v86_far a_Params; \
+    a_RetType a_Name##_pae32_16_far  a_Params; \
+    a_RetType a_Name##_paev86_far    a_Params; \
+    a_RetType a_Name##_lm16_far      a_Params
+#define BS3_MODE_EXPAND_EXTERN_DATA16(a_VarType, a_VarName, a_Suffix) \
+    extern a_VarType a_VarName##_rm        a_Suffix; \
+    extern a_VarType a_VarName##_pe16      a_Suffix; \
+    extern a_VarType a_VarName##_pe16_32   a_Suffix; \
+    extern a_VarType a_VarName##_pe16_v86  a_Suffix; \
+    extern a_VarType a_VarName##_pe32      a_Suffix; \
+    extern a_VarType a_VarName##_pe32_16   a_Suffix; \
+    extern a_VarType a_VarName##_pev86     a_Suffix; \
+    extern a_VarType a_VarName##_pp16      a_Suffix; \
+    extern a_VarType a_VarName##_pp16_32   a_Suffix; \
+    extern a_VarType a_VarName##_pp16_v86  a_Suffix; \
+    extern a_VarType a_VarName##_pp32      a_Suffix; \
+    extern a_VarType a_VarName##_pp32_16   a_Suffix; \
+    extern a_VarType a_VarName##_ppv86     a_Suffix; \
+    extern a_VarType a_VarName##_pae16     a_Suffix; \
+    extern a_VarType a_VarName##_pae16_32  a_Suffix; \
+    extern a_VarType a_VarName##_pae16_v86 a_Suffix; \
+    extern a_VarType a_VarName##_pae32     a_Suffix; \
+    extern a_VarType a_VarName##_pae32_16  a_Suffix; \
+    extern a_VarType a_VarName##_paev86    a_Suffix; \
+    extern a_VarType a_VarName##_lm16      a_Suffix; \
+    extern a_VarType a_VarName##_lm32      a_Suffix; \
+    extern a_VarType a_VarName##_lm64      a_Suffix
+
 EOF
 
     MY_HDR_FILES=`  echo ${MY_ROOT_DIR}/include/VBox/*.h ${MY_ROOT_DIR}/include/VBox/vmm/*.h \
@@ -829,6 +1026,8 @@ my_generate_project "VMM"           "src/VBox/VMM"                          --be
     "include/VBox/vmm/cpum.*" \
     "include/VBox/vmm/dbgf.h" \
     "include/VBox/vmm/em.h" \
+    "include/VBox/vmm/gim.h" \
+    "include/VBox/vmm/apic.h" \
     "include/VBox/vmm/gmm.*" \
     "include/VBox/vmm/gvm.*" \
     "include/VBox/vmm/hm*.*" \
@@ -921,6 +1120,7 @@ my_generate_project "SUP"           "src/VBox/HostDrivers/Support"          --be
 my_generate_project "VBoxNetAdp"    "src/VBox/HostDrivers/VBoxNetAdp"       --begin-incs "include" "src/VBox/HostDrivers/VBoxNetAdp"        --end-includes "src/VBox/HostDrivers/VBoxNetAdp" "include/VBox/intnet.h"
 my_generate_project "VBoxNetFlt"    "src/VBox/HostDrivers/VBoxNetFlt"       --begin-incs "include" "src/VBox/HostDrivers/VBoxNetFlt"        --end-includes "src/VBox/HostDrivers/VBoxNetFlt" "include/VBox/intnet.h"
 my_generate_project "VBoxUSB"       "src/VBox/HostDrivers/VBoxUSB"          --begin-incs "include" "src/VBox/HostDrivers/VBoxUSB"           --end-includes "src/VBox/HostDrivers/VBoxUSB" "include/VBox/usblib*.h" "include/VBox/usbfilter.h"
+my_generate_project "AdpCtl"        "src/VBox/HostDrivers/adpctl"           --begin-incs "include"                                          --end-includes "src/VBox/HostDrivers/adpctl"
 
 # src/VBox/HostServices
 my_generate_project "GuestCntl"     "src/VBox/HostServices/GuestControl"    --begin-incs "include" "src/VBox/HostServices/GuestControl"     --end-includes "src/VBox/HostServices/GuestControl"
@@ -951,23 +1151,24 @@ my_generate_project "RDP-Server"    "src/VBox/RDP/server"                   --be
 my_generate_project "RDP-WebClient" "src/VBox/RDP/webclient"                --begin-incs "include" "src/VBox/RDP/webclient"                 --end-includes "src/VBox/RDP/webclient"
 my_generate_project "RDP-Misc"      "src/VBox/RDP"                          --begin-incs "include"                                          --end-includes "src/VBox/RDP/auth" "src/VBox/RDP/tscpasswd" "src/VBox/RDP/x11server"
 
+# src/VBox/Storage
+my_generate_project "Storage"       "src/VBox/Storage"                      --begin-incs "include" "src/VBox/Storage"                       --end-includes "src/VBox/Storage"
+
 # src/VBox/ValidationKit
 my_generate_project "ValidationKit" "src/VBox/ValidationKit"                --begin-incs "include"                                          --end-includes "src/VBox/ValidationKit"
 
 # src/VBox/ExtPacks
 my_generate_project "ExtPacks"      "src/VBox/ExtPacks"                     --begin-incs "include"                                          --end-includes "src/VBox/ExtPacks"
 
-# src/apps/adpctl - misplaced.
-my_generate_project "adpctl"        "src/apps/adpctl"                       --begin-incs "include"                                          --end-includes "src/apps/adpctl"
-
 # src/bldprogs
 my_generate_project "bldprogs"      "src/bldprogs"                          --begin-incs "include"                                          --end-includes "src/bldprogs"
 
 # A few things from src/lib
-my_generate_project "zlib"          "src/libs/zlib-1.2.8"                   --begin-incs "include"                                          --end-includes "src/libs/zlib-1.2.8/*.c" "src/libs/zlib-1.2.6/*.h"
+my_generate_project "zlib"          "src/libs/zlib-1.2.8"                   --begin-incs "include"                                          --end-includes "src/libs/zlib-1.2.8/*.c" "src/libs/zlib-1.2.8/*.h"
 my_generate_project "liblzf"        "src/libs/liblzf-3.4"                   --begin-incs "include"                                          --end-includes "src/libs/liblzf-3.4"
-my_generate_project "libpng"        "src/libs/libpng-1.2.54"                --begin-incs "include"                                          --end-includes "src/libs/libpng-1.2.54/*.c" "src/libs/libpng-1.2.8/*.h"
-my_generate_project "openssl"       "src/libs/openssl-1.0.1t"               --begin-incs "include" "src/libs/openssl-1.0.1t/crypto"         --end-includes "src/libs/openssl-1.0.1t"
+my_generate_project "libpng"        "src/libs/libpng-1.2.54"                --begin-incs "include"                                          --end-includes "src/libs/libpng-1.2.54/*.c" "src/libs/libpng-1.2.54/*.h"
+my_generate_project "openssl"       "src/libs/openssl-1.1.0b"               --begin-incs "include" "src/libs/openssl-1.1.0b/crypto"         --end-includes "src/libs/openssl-1.1.0b"
+my_generate_project "curl"          "src/libs/curl-7.50.3"                  --begin-incs "include" "src/libs/curl-7.50.3/include"           --end-includes "src/libs/curl-7.50.3"
 my_generate_project "kStuff"        "src/libs/kStuff"                       --begin-incs "include" "src/libs/kStuff/kStuff/include"         --end-includes "src/libs/kStuff"
 
 
@@ -1037,4 +1238,3 @@ my_generate_usercpp_h
 
 
 echo "done"
-
